// ============================================================================
// Project:	Deductive Database
// Filename:	atomtab.h
// Purpose:	Atom Table (stores set of atoms, similar to Enumeration Type)
// Last Change:	24.05.2016
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2010-2016 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	ATOMTAB_INCLUDED
#define ATOMTAB_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef MPOOL_INCLUDED
#include "mpool.h"
#endif

#ifndef FLEXARR_INCLUDED
#include "flexarr.h"
#endif

#ifndef ATOM_INCLUDED
#include "atom.h"
#endif


//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// ATOMTAB_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long ATOMTAB_MAGIC = 0x4154420AL;	// 'ATB\n'

//-----------------------------------------------------------------------------
// ATOMTAB_HASHSIZE: Size of hash table.
//-----------------------------------------------------------------------------

static const int ATOMTAB_HASHSIZE = 251; // Should be prime number

//=============================================================================
// Class for Atom Table (stores set of atoms, similar to Enumeration Type):
//=============================================================================

class atomtab_c {
	public:

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	// Constructor:
	atomtab_c(str_t tab_name);

	// Destructor:
	~atomtab_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// put: Search current token in atom table, insert if not found.
	int put(str_t name_start, str_t name_end);

	// put: As above, but with '\0'-terminated string:
	int put(str_t atom_name) {
		CHECK_VALID("atomtab_c::put(str_t)");
		CHECK(atom_name != STR_NULL,
				"atomtab_c::put(str_t): atom_name is null");
		return put(atom_name, atom_name + str_len(atom_name));
	}

	// get: Return atom name for given id:
	str_t get(int id);

	// name: Get name of this atom table (similar to enumeration type).
	str_t name() const {
		CHECK_VALID("atomtab_c::name");
		return Name;
	}

	// num_atoms: Get number of atoms in this table.
	int num_atoms() const {
		CHECK_VALID("atomtab_c::num_atoms");
		return NumAtoms;
	}

	// Return number of memory pages:
	int num_pages() const {
		return MemPool.num_alloc_pages();
	}

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	public:
	// Integrity check:
	str_t	check() const;

	private:
	// Magic number (identifies objects of this class for debugging).
	long	Magic;		// Must be "ATOMTAB_MAGIC".
#endif

#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	atomtab_c(const atomtab_c& obj);		// Not implemented
	atomtab_c& operator=(const atomtab_c& obj);	// Not implemented

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

	// Memory Pool for Allocating Storage Pages:
	mpool_c MemPool;

	// Current Memory Page for Storing Atom Objects:
	page_t MemPage;

	// Pointer to next free place in current memory page:
	atom_t MemPtr;

	// Number of free places in current page:
	int MemFree;

	// Hash table:
	atom_t HashTab[ATOMTAB_HASHSIZE];

	// Name of this atom table (domain, enumeration type):
	str_t Name;

	// Current number of atoms (also Id of next atom):
	int NumAtoms;

	// Find atom by id, small (standard) version:
	atom_t AtomArr[ATOMTAB_HASHSIZE];

	// Find atom by id, large version:
	flexarr_c<atom_t> LargeAtomArr;

//=============================================================================
// End of Class:
//=============================================================================

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef atomtab_c *atomtab_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define ATOMTAB_NULL (static_cast<atomtab_t>(0))

//=============================================================================
// End of Include File:
//=============================================================================

#endif

